import java.io.*;
import java.util.*;

class SKIX {
  static final int S = 0;
  static final int K = 1;
  static final int I = 2;
  static final int X = 3;
  static final int A = 4;
  static public long work = 0;
  static public boolean trace = false;;

  SKIX left = null;
  SKIX right = null;
  int kind = 0;

  public SKIX(int k) {
    kind = k;
  }

  public SKIX(SKIX l, SKIX r) {
    kind = A;
    left = l;
    right = r;
  }

  public String toString() {
    if (kind==S) return "S";
    if (kind==K) return "K";
    if (kind==I) return "I";
    if (kind==X) return "X";
    if (left.kind==A && right.kind==A)
      return "("+left+")("+right+")"; 
    if (left.kind!=A && right.kind==A)
      return left+"("+right+")"; 
    if (left.kind==A && right.kind!=A)
      return "("+left+")"+right;
    return ""+left+right;
  }  

  public int toBool() {
    if (kind==K) return 1;
    if (kind!=A) return -1;
    String s = toString();
    if (s.equals("SK")) return 0;
    return -1;
  }

  public int toNum() {
    if (kind!=A) return -1;
    String s = toString();
    if (s.equals("KI") || s.equals("K((SK)K)")) return 0;
    int i = right.toNum();
    if (i!=-1 && left.toString().equals("S((S(KS))K)")) 
      return i+1;
    return -1;
  }

  static public SKIX zero = parse("KI");
  static public SKIX succ = parse("S((S(KS))K)");
  static public SKIX pred = parse("S(K(SI(K(KI))))(S(SI(K(S(K(S(K(S(SI(K(KI)))))K))(S(S(SI(KK))(SI(K(KI))))(S(K(S(S(KS)K)))(SI(K(KI))))))))(K(S(SI(KK))(K(KI)))))");
  static public SKIX iszero = parse("((S(SI(K(K(SK))))(KK))");
  static public SKIX not = parse("S(SI(K(SK)))(KK)");
  static public SKIX and = parse("S(S(KS)I)(K(K(SK)))");
  static public SKIX fix = parse("S(K(S(I)(I)))(S(S(K(S))(K))(K(S(I)(I))))");
  static public SKIX plus = parse("SI(K(S(S(KS)K)))");
  static public SKIX mult = parse("S(S(KS)(S(K(SI))(S(KK)(SI(K(S(S(KS)K)))))))(K(K(KI)))");
  static public SKIX tt = parse("K");
  static public SKIX ff = parse("SK");

  public static SKIX mkNum(int i) {
    if (i==0) return zero;
    return new SKIX(succ,mkNum(i-1));
  }

  public SKIX toX() {
    if (kind==A) {
      left.toX();
      right.toX();
    }
    else if (kind==S) {
      kind = A;
      left = new SKIX(X);
      right = new SKIX(new SKIX(X),new SKIX(X));
    }
    else if (kind==K) {
      kind = A;
      left = new SKIX(new SKIX(X),new SKIX(X));
      right = new SKIX(X);
    }
    else if (kind==I) {
     kind = A;
     left = new SKIX(new SKIX(S),new SKIX(K));;
     right = new SKIX(K);
     toX();
    }
    return this;
  }

  public Object clone() {
    if (kind==A) return new SKIX((SKIX)left.clone(),(SKIX)right.clone());
    return new SKIX(kind);
  }

  static char next = ' ';
  static String input = "";
  static int index = 0;

  static void setParse(String s) {
    input = s;
    index = 0;
    skip();
  }

  static void skip() {
    while (index<input.length() && (input.charAt(index)==' ' || input.charAt(index)=='\n')) index++;
    if (index<input.length()) next = input.charAt(index);
  } 

  static boolean done() {
    return index>=input.length();
  }

  static SKIX parse() {
    SKIX e = null;
    SKIX p = null;
    while (!done() && next!=')') {
      if (next=='(') {
        index++;
        skip();
        p = parse();
      }
      if (next=='S') p = new SKIX(S);
      if (next=='K') p = new SKIX(K);
      if (next=='I') p = new SKIX(I);
      if (next=='X') p = new SKIX(X);
      if ('a'<=next && next<='z') {
       String s = "";
       while ('a'<=next && next<='z') {
         s = s+next;
         if (index>=input.length()) break;
         index++;
         next = input.charAt(index); 
       } 
       if (s.equals("true"))  p =tt;
       if (s.equals("false"))  p =ff;
       if (s.equals("pred"))  p =pred;
       if (s.equals("zero"))  p =zero;
       if (s.equals("succ"))  p =succ;
       if (s.equals("iszero"))  p =iszero;
       if (s.equals("not"))  p =not;
       if (s.equals("and"))  p =and;
       if (s.equals("fix"))  p =fix;
       if (s.equals("plus"))  p =plus;
       if (s.equals("mult"))  p =mult;
      } 
      else if ('0'<=next && next<='9') {
       int n = 0;
       while ('0'<=next && next<='9') {
         n = 10*n+next-(int)'0';
         if (index>=input.length()) break;
         index++;
         next = input.charAt(index); 
       } 
       p = mkNum(n);
     }
     else index++;
     skip();
     if (e==null) e = p;
     else e = new SKIX(e,p);
    }
    return e;
  }

  public static SKIX parse(String s) {
    setParse(s);
    return parse();
  }

  boolean match(int k, int n) {
    SKIX e = this;
    for (int i=0; i<n; i++) {
      if (e.kind!=A) return false;
      e = e.left;
    }
    return e.kind==k;
  }

  boolean reduceX() {
    SKIX a = right;
    right = new SKIX(K);
    left = new SKIX(new SKIX(a,new SKIX(K)),new SKIX(S));
    work++;
    return true;
  }

  boolean reduceS() {
    SKIX a = left.left.right;
    SKIX b = left.right;
    SKIX c = right;
    left = new SKIX(a,c);
    right = new SKIX(b,c);
    work++;
    return true;
  }

  boolean reduceK() {
    kind = left.right.kind; 
    right = left.right.right;
    left = left.right.left;
    work++;
    return true;
  }

  boolean reduceI() {
    kind = right.kind;
    left = right.left;
    right = right.right;
    work++;
    return true;
  }

  public boolean reduce() {
    if (kind!=A) return false;
    if (match(X,1)) return reduceX();
    if (match(S,3)) return reduceS();
    if (match(K,2)) return reduceK();
    if (match(I,1)) return reduceI();
    return left.reduce() || right.reduce();
  }

  public long normalize() {
    work = 0;
    while (reduce()) {
      if (trace) System.err.println(this);
    }
    return work;
  }
}

public class X {

  static String stdin() {
    StringBuffer br = new StringBuffer();
    int i;
    try {
      while ((i=System.in.read())!=-1) br.append((char)i);
    } catch(IOException e) {
    }
    return br.toString(); 
  }
 
  public static void main(String[] args) {
    String s = args[0];
    if (s.equals("translate")) {
      System.out.println(SKIX.parse(stdin()).toX());
    }
    if (s.equals("run") || s.equals("trace")) {
      if (s.equals("trace")) SKIX.trace = true;
      SKIX e = SKIX.parse(stdin());
      long work = e.normalize();
      System.out.println(e);
      System.err.println("Work = "+work);
    }
    if (s.equals("constant")) {
      SKIX e = SKIX.parse(stdin());
      int b = e.toBool();
      if (b==0) {       
        System.out.println("false"); System.exit(0);
     }
      if (b==1) {       
        System.out.println("true"); System.exit(0);
     }
      e = new SKIX(new SKIX(e,SKIX.succ),SKIX.zero);
      e.normalize();
      int r = e.toNum();
      if (r==-1)
        System.out.println("Not a numeral or boolean");
      else
        System.out.println(r);
    }
  }
}
